/* USER CODE BEGIN 1 */
/*
 * HAL_layer_io.h
 *
 *  Created on: 10. 3. 2018
 *      Author: Dušan Benda
 */

#ifndef HAL_LAYER_IO_H_
#define HAL_LAYER_IO_H_

#include "HAL_layer_LED.h"
#include "HAL_layer_io.h"
#include "HAL_layer_protection.h"

void HAL_MCU_fault(uint8_t state);
uint8_t HAL_Switch_0(void);
uint8_t HAL_Switch_1(void);
uint8_t HAL_Switch_2(void);
uint8_t HAL_Switch_3(void);
uint8_t HAL_Switch_4(void);
uint8_t HAL_Switch_5(void);
uint8_t HAL_Switches(void);
uint16_t HAL_fault_io_check(void);

void HAL_LT3992_power_12V_SHDN_on(void);
void HAL_LT3992_power_12V_SHDN_off(void);
void HAL_LT3992_power_12V_SHDN(uint8_t state);


#endif /* HAL_LAYER_IO_H_ */

/* USER CODE END 1 */
